from urllib3.connectionpool import VerifiedHTTPSConnection
from .stubs import VCRHTTPSConnection

class VCRVerifiedHTTPSConnection(VCRHTTPSConnection, VerifiedHTTPSConnection):

    _baseclass = VerifiedHTTPSConnection




import httplib
from contextlib import contextmanager
from .stubs import VCRHTTPConnection, VCRHTTPSConnection


_HTTPConnection = httplib.HTTPConnection
_HTTPSConnection = httplib.HTTPSConnection

try:
    import requests.packages.urllib3.connectionpool
    _VerifiedHTTPSConnection = requests.packages.urllib3.connectionpool.VerifiedHTTPSConnection 
except ImportError:
    pass

try:
    import urllib3
    _VerifiedHTTPSConnection = urllib3.connectionpool.VerifiedHTTPSConnection 
except ImportError:
    pass


def install(cassette_path):
    httplib.HTTPConnection = httplib.HTTP._connection_class = VCRHTTPConnection
    httplib.HTTPSConnection = httplib.HTTPS._connection_class = VCRHTTPSConnection
    httplib.HTTPConnection._vcr_cassette_path = cassette_path
    httplib.HTTPSConnection._vcr_cassette_path = cassette_path

    #patch requests
    try:
        import requests.packages.urllib3.connectionpool
        from .requests_stubs import VCRVerifiedHTTPSConnection
        requests.packages.urllib3.connectionpool.VerifiedHTTPSConnection = VCRVerifiedHTTPSConnection
        requests.packages.urllib3.connectionpool.VerifiedHTTPSConnection._vcr_cassette_path = cassette_path
    except ImportError:
        pass

    #patch urllib3
    try:
        import urllib3.connectionpool
        from .urllib3_stubs import VCRVerifiedHTTPSConnection
        urllib3.connectionpool.VerifiedHTTPSConnection = VCRVerifiedHTTPSConnection
        urllib3.connectionpool.VerifiedHTTPSConnection._vcr_cassette_path = cassette_path
        urllib3.connectionpool.HTTPConnection = VCRHTTPConnection
        urllib3.connectionpool.HTTPConnection._vcr_cassette_path = cassette_path
    except ImportError:
        pass


def reset():
    httplib.HTTPConnection = httplib.HTTP._connection_class = _HTTPConnection
    httplib.HTTPSConnection = httplib.HTTPS._connection_class = \
            _HTTPSConnection
    try:
        import requests.packages.urllib3.connectionpool
        requests.packages.urllib3.connectionpool.VerifiedHTTPSConnection = _VerifiedHTTPSConnection
    except ImportError:
        pass

    try:
        import urllib3.connectionpool
        urllib3.connectionpool.VerifiedHTTPSConnection = _VerifiedHTTPSConnection
        urllib3.connectionpool.HTTPConnection = _HTTPConnection
    except ImportError:
        pass


@contextmanager
def use_cassette(cassette_path):
    install(cassette_path)
    yield
    reset()



from httplib import HTTPConnection, HTTPSConnection, HTTPMessage
from cStringIO import StringIO
from .files import save_cassette, load_cassette
from .cassette import Cassette


class VCRHTTPResponse(object):
    """
    Stub reponse class that gets returned instead of a HTTPResponse
    """
    def __init__(self, recorded_response):
        self.recorded_response = recorded_response
        self.reason = recorded_response['status']['message']
        self.status = recorded_response['status']['code']
        self.version = None
        self._content = StringIO(self.recorded_response['body']['string'])

        self.msg = HTTPMessage(StringIO(''))
        for k, v in self.recorded_response['headers'].iteritems():
            self.msg.addheader(k, v)

        self.length = self.msg.getheader('content-length') or None

    def read(self, chunked=False):
        # Note: I'm pretty much ignoring any chunking stuff because
        # I don't really understand what it is or how it works.
        return self._content.read()

    def isclosed(self):
        # Urllib3 seems to call this because it actually uses
        # the weird chunking support in httplib
        return True

    def getheaders(self):
        return self.recorded_response['headers'].iteritems()


class VCRConnectionMixin:
    def _load_old_response(self):
        old_cassette = load_cassette(self._vcr_cassette_path)
        if old_cassette:
            return old_cassette.get_response(self._vcr)

    def request(self, method, url, body=None, headers={}):
        """
        Persist the request metadata in self._vcr
        """
        self._vcr = {
            'method': method,
            'url': url,
            'body': body,
            'headers': headers,
        }
        old_cassette = load_cassette(self._vcr_cassette_path)
        if old_cassette and old_cassette.get_request(self._vcr):
            return
        self._cassette.requests.append(dict(
            method=method,
            url=url,
            body=body,
            headers=headers
        ))
        self._baseclass.request(self, method, url, body=body, headers=headers)

    def getresponse(self, buffering=False):
        old_response = self._load_old_response()
        if not old_response:
            response = HTTPConnection.getresponse(self)
            self._cassette.responses.append({
                'status': {'code': response.status, 'message': response.reason},
                'headers': dict(response.getheaders()),
                'body': {'string': response.read()},
            })
            save_cassette(self._vcr_cassette_path, self._cassette)
        old_response = self._load_old_response()
        return VCRHTTPResponse(old_response)


class VCRHTTPConnection(VCRConnectionMixin, HTTPConnection):

    # Can't use super since this is an old-style class
    _baseclass = HTTPConnection

    def __init__(self, *args, **kwargs):
        self._cassette = Cassette()
        HTTPConnection.__init__(self, *args, **kwargs)


class VCRHTTPSConnection(VCRConnectionMixin, HTTPSConnection):

    _baseclass = HTTPSConnection

    def __init__(self, *args, **kwargs):
        """
        I overrode the init and copied a lot of the code from the parent
        class because HTTPConnection when this happens has been replaced
        by VCRHTTPConnection,  but doing it here lets us use the original
        one.
        """
        HTTPConnection.__init__(self, *args, **kwargs)
        self.key_file = kwargs.pop('key_file', None)
        self.cert_file = kwargs.pop('cert_file', None)
        self._cassette = Cassette()



import os
import yaml
from .cassette import Cassette


def load_cassette(cassette_path):
    try:
        pc = yaml.load(open(cassette_path))
        cassette = Cassette(pc)
        return cassette
    except IOError:
        return None


def save_cassette(cassette_path, cassette):
    dirname, filename = os.path.split(cassette_path)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    with open(cassette_path, 'a') as cassette_file:
        cassette_file.write(yaml.dump(cassette.serialize()))



class Cassette(object):
    def __init__(self, ser_cassette=None):
        self.requests = []
        self.responses = []
        if ser_cassette:
            self._unserialize(ser_cassette)

    def serialize(self):
        return ([{
            'request': req,
            'response': res,
        }  for req, res in zip(self.requests, self.responses)])

    def _unserialize(self, source):
        self.requests, self.responses = [r['request'] for r in source], [r['response'] for r in source]

    def get_request(self, match):
        try:
            return self.requests[self.requests.index(match)]
        except ValueError:
            return None

    def get_response(self, match):
        try:
            return self.responses[self.requests.index(match)]
        except ValueError:
            return None



#!/usr/bin/env python

import sys
from setuptools import setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)

setup(name='vcrpy',
      version='0.0.1',
      description="A Python port of Ruby's VCR to make mocking HTTP easier",
      author='Kevin McCarthy',
      author_email='me@kevinmccarthy.org',
      url='https://github.com/kevin1024/vcrpy',
      packages=['vcr'],
      install_requires=['PyYAML'],
      license='MIT',
      tests_require=['pytest'],
      cmdclass={'test': PyTest},
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Programming Language :: Python',
          'Topic :: Software Development :: Testing',
          'Topic :: Internet :: WWW/HTTP',
          'License :: OSI Approved :: MIT License',
      ],
)
